<?php

/**
 * @file
 * Import photos and other data from Gallery2.
 *
 * Other parts of this module import data from Gallery2 SQL dumps and add data
 * to custom tables. This file uses the Migrate API to describe how to create
 * Drupal objects (nodes, taxonomy terms, etc.) from these data and the actual
 * photos.
 */

/**
 * Create taxonomy terms from the album structure.
 */
class Gallery2AlbumMigration extends Migration {
  public function __construct() {
    parent::__construct();
    $this->description = t('Create taxonomy terms from the Gallery2 album structure.');

    $query = db_select('g2migrate_Album', 'a');
    $query->join('g2migrate_ChildEntity', 'c', 'a.g_id = c.g_id');
    $query->join('g2migrate_Item', 'i', 'a.g_id = i.g_id');
    $query->fields('a', array('g_id'))
      ->fields('c', array('g_parentid'))
      ->fields('i', array('g_title', 'g_summary'))
      ->orderBy('depth', 'ASC');
    $this->source = new MigrateSourceSQL($query);

    // Import terms from album names in the g2migrate_Album table.
    $this->map = new MigrateSQLMap($this->machineName,
      array(
        'g_id' => array(
          'description' => 'The primary key from the {g2migrate_Entity} table.',
          'type' => 'int',
          'size' => 'big',
          'not null' => TRUE,
          'alias' => 'a',
        ),
      ),
      MigrateDestinationTerm::getKeySchema()
    );

    // Create terms in the g2migrate_album vocabulary.
    $vocabulary = variable_get('g2migrate_vocabulary', 'g2migrate_album');
    $this->destination = new MigrateDestinationTerm($vocabulary);

    // The order of the arguments is target, source, as in $target = $source.
    $this->addFieldMapping('name', 'g_title');
    $this->addFieldMapping('description', 'g_summary');
    $this->addFieldMapping('parent', 'g_parentid')
      ->sourceMigration('Gallery2Album');
    $this->addUnmigratedDestinations(
      array('format', 'weight', 'parent_name')
    );

    // We conditionally DNM these fields, so your field mappings will be clean
    // whether or not you have path and or pathauto enabled
    if (module_exists('path')) {
      $this->addFieldMapping('path')
           ->issueGroup(t('DNM'));
      if (module_exists('pathauto')) {
        $this->addFieldMapping('pathauto')
             ->issueGroup(t('DNM'));
      }
    }

  }
}

/**
 * Gallery2 base class.
 *
 * The field mappings will depend on whether we create nodes (using the core
 * Image field) or Media entities. This class handles the source data, common to
 * both.
 */
abstract class Gallery2BaseMigration extends Migration {
  public function __construct() {
    parent::__construct();

    // The Album vocabulary holds the album structure from Gallery2. Make sure
    // to import that first, so that now we can attach terms to photos. 
    $this->dependencies = array('Gallery2Album');

    $query = db_select('g2migrate_Entity', 'e');
    $query->join('g2migrate_Item', 'i', 'e.g_id = i.g_id');
    $query->join('g2migrate_ChildEntity', 'c', 'e.g_id = c.g_id');
    $query->join('g2migrate_Album', 'a', 'c.g_parentid = a.g_id');
    $query->join('g2migrate_FileSystemEntity', 'f', 'e.g_id = f.g_id');
    $query->condition('g_entityType', 'GalleryPhotoItem')
      ->fields('e', array('g_id'))
      ->fields('i', array('g_title', 'g_keywords', 'g_summary'))
      ->fields('c', array('g_parentid'))
      ->addExpression('CONCAT(a.path, \'/\', f.g_pathComponent)', 'path');
    $this->source = new MigrateSourceSQL($query);

  }
}

/**
 * The moment you have been waiting for: import those photos.
 *
 * This class handles photos attached using the core Image field. Use
 * Gallery2Media1NodeMigration instead if you are using the Media module.
 */
class Gallery2NodeMigration extends Gallery2BaseMigration {
  public function __construct() {
    parent::__construct();
    $this->description = t('Create nodes and attach photos as Image fields from the Gallery2 album.');

    $this->map = new MigrateSQLMap($this->machineName,
      array(
        'g_id' => array(
          'description' => 'size was 11',
          'type' => 'int',
          'size' => 'big',
          'not null' => TRUE,
          'alias' => 'e',
        ),
      ),
      MigrateDestinationNode::getKeySchema()
    );

    $node_type = variable_get('g2migrate_node_type', 'g2migrate_photo');
    $this->destination = new MigrateDestinationNode($node_type);

    // TODO: Keep track of owners.
    $this->addFieldMapping('uid')
      ->defaultValue(1);

    // Do not promote to the front page, and do not make sticky.
    $this->addFieldMapping('promote')
      ->defaultValue(0);
    $this->addFieldMapping('sticky')
      ->defaultValue(0);

    // For each key in $sources, there is a corresponding persistent variable
    // holding the name of the field to use. Other data are stored in the
    // $sources array rather than here to avoid defining two arrays and keeping
    // them synchronized.
    $unmigrated = array(
      'created', 'changed', 'status',
      'revision', 'language', 'revision_uid', 'log', 'tnid',
      'body:format', 'body:language', 'body:summary',
      'comment', 'is_new',
    );
    foreach ($sources = _g2migrate_sources() as $key => $data) {
      $field = variable_get("g2migrate_field_$key", $data['default']);
      if ($field == 'none') {
        continue;
      }
      // $sources[$key]['field'] = $field;
      $field_mapping = $this->addFieldMapping($field, $key);
      if (array_key_exists('description', $data)) {
        $field_mapping = $field_mapping->description($data['description']);
      }
      if (array_key_exists('sourceMigration', $data)) {
        $field_mapping = $field_mapping->sourceMigration($data['sourceMigration']);
      }
      if (array_key_exists('defaultValue', $data)) {
        $field_mapping = $field_mapping->defaultValue($data['defaultValue']);
      }
      if (array_key_exists('separator', $data)) {
        $field_mapping = $field_mapping->separator($data['separator']);
      }
      if (array_key_exists('unmigrated', $data)) {
        foreach ($data['unmigrated'] as $subfield) {
          $unmigrated[] = "$field:$subfield";
        }
      }
    }
    // $this->addFieldMapping('body:summary', 'excerpt');
    // dsm($sources);

    // The first vocabulary will hold the terms based on the Gallery2 album
    // structure.  The second holds keywords.
    $vocabulary = variable_get('g2migrate_field_g_parentid', 'g2migrate_album');
    $tags = variable_get('g2migrate_field_g_keywords', 'g2migrate_keywords');
    if ($vocabulary != 'none') {
      $this->addFieldMapping("$vocabulary:source_type")
        ->defaultValue('tid');
    }
    if ($tags != 'none') {
      $this->addFieldMapping("$tags:create_term")
        ->defaultValue(TRUE);
    }

    // Unmapped destination fields
    $this->addUnmigratedDestinations($unmigrated);

    // These fields are supported by the core image field but not by Media.
    $field = variable_get('g2migrate_field_path', 'g2migrate_image');
    // $info = field_info_field($field);
    $this->addFieldMapping("$field:source_dir")
      ->defaultValue(variable_get('g2migrate_g2data') . '/albums');
    $this->addFieldMapping("$field:alt", 'g_title');
    $this->addFieldMapping("$field:title", 'g_title');

    if (module_exists('path')) {
      $this->addFieldMapping('path')
        ->issueGroup(t('DNM'));
      if (module_exists('pathauto')) {
        $this->addFieldMapping('pathauto')
          ->issueGroup(t('DNM'));
      }
    }
    if (module_exists('statistics')) {
      $this->addUnmigratedDestinations(array('totalcount', 'daycount', 'timestamp'));
    }

  }
}
